import config
import message
from core.checker import BaseChecker, CheckerManager, share
from core.textparser import TextParser


class CPU(BaseChecker):

    def __init__(self):
        super().__init__("CPU规格")
        self.initial_info_collection_file()

    @share
    def cores(self):
        out = self.cmd("grep -c 'processor' /proc/cpuinfo")
        return int(out.stdout.strip())

    def run(self) -> None:
        cores = self.cores()
        if cores < config.cpu_min_vcore:
            self.alarms.add(message.核数过少.format(cores, config.cpu_min_vcore))


class Top(BaseChecker):

    def __init__(self):
        super().__init__('CPU性能')
        self.initial_info_collection_file()

    @share
    def top(self):
        # top第一次不准确，因此这里取第二次输出
        out = self.cmd("top -b -n 2 -d 0.01 | awk '/^top/{i++}i==2'")
        parser = TextParser(
            out.stdout,
            strip_tail='PID{:s}USER{:s}PR{:s}NI{:s}VIRT{:s}RES{:s}SHR{:s}S{:s}%CPU{:s}%MEM{:s}TIME+{:s}COMMAND')
        top_header = parser.parse(
            'load average: {ld5:f}, {ld10:f}, {ld15:f}',
            '%Cpu(s):{:s}{cpu_us:f} us,{:s}{cpu_sy:f} sy,{:s}{:f} ni,{:s}{:f} id,{:s}{cpu_wa:f} wa',
            'Cpu(s):{:s}{cpu_us:f}%us,{:s}{cpu_sy:f}%sy,{:s}{:f}%ni,{:s}{:f}%id,{:s}{cpu_wa:f}%wa',
            full_match=False)
        return top_header

    def run(self) -> None:
        header = self.top()
        ld5, ld10, ld15 = header['ld5'], header['ld10'], header['ld15']
        vcores = CheckerManager.share(CPU, 'cores')
        if ld5 > vcores or ld10 > vcores or ld15 > vcores:
            self.alarms.add(message.负载高.format(ld5, ld10, ld15, vcores))
        sys = header['cpu_sy']
        if sys > config.cpu_max_sys:
            self.alarms.add(message.内核CPU占用高.format(sys, config.cpu_max_sys))
        wa = header['cpu_wa']
        if wa > config.cpu_max_wa:
            self.alarms.add(message.CPU_IO等待时间高.format(wa, config.cpu_max_wa))
        us = header['cpu_us']
        if us > config.cpu_max_user:
            self.alarms.add(message.用户CPU占用高.format(us, config.cpu_max_user))
