import message
from core.checker import BaseChecker, share
from core.textparser import TextParser
from core.common import del_empty


class Greenplum(BaseChecker):

    def __init__(self):
        super().__init__("greenplum")

    @share
    def tasks(self, passwd) -> list:
        """
        只在主节点上执行该方法
        """
        if self.should_run() and passwd.strip() != '':
            self.initial_info_collection_file()
            out = self.gpsql(
                'postgres', passwd,
                "select *, ((now()-query_start)/'1 minute')::integer AS elapsed_hours from pg_stat_activity where state='active'")
            parser = TextParser(
                out.stdout,
                strip_header='-------',
                strip_tail='({:d} row'
            )
            tasks = parser.parse_rows_by_spilt('|',
                                               ['datid', 'datname', 'pid', 'sess_id', 'usesysid', 'usename',
                                                'application_name', 'client_addr',
                                                'client_hostname', 'client_port', 'backend_start', 'xact_start',
                                                'query_start', 'state_change', 'waiting',
                                                'state', 'backend_xid', 'backend_xmin', 'query', 'waiting_reason', 'rsgid',
                                                'rsgname', 'rsgqueueduration',
                                                'elapsed_hours'])

            return tasks

        return []

    @share
    def gp_passwd(self) -> str:
        """
        只在主节点上执行该方法
        """
        if self.should_run():
            self.initial_info_collection_file()
            out = self.cmd(
                'unipd_64 1 `cat /bigdata/salut/components/jetty/resources/application.properties | grep greenplum.password | cut -d= -f 2`')
            return del_empty(out.stdout)

        return ''

    @share
    def down_nodes(self, passwd) -> list:
        """
        只在主节点上执行该方法
        """
        if self.should_run() and passwd.strip() != '':
            self.initial_info_collection_file()
            out = self.gpsql(
            'postgres', passwd,
            "select * from gp_segment_configuration where status= 'd'")
            parser = TextParser(
                out.stdout,
                strip_header='-------',
                strip_tail='({:d} row')
            downs = parser.parse_rows_by_spilt('|',
                                               ['dbid', 'content', 'role', 'preferred_role', 'mode', 'status', 'port',
                                                'hostname', 'address', 'datadir'])
            return downs

        return []

    def run(self) -> None:
        passwd = self.gp_passwd()
        down_nodes = self.down_nodes(passwd)
        tasks = self.tasks(passwd)
        if down_nodes and len(down_nodes) > 0:
            self.alarms.add(message.GP集群不健康)
        for t in tasks:
            elapsed_hours = int(t['elapsed_hours'])
            if elapsed_hours > 1:
                self.alarms.add(message.GP长任务.format(elapsed_hours, t['query'], t['pid'], t['pid']))
