import message
from core.checker import BaseChecker, share
from core.textparser import TextParser


class HDFS(BaseChecker):

    def __init__(self):
        super().__init__('HDFS')

    @share
    def fsck(self) -> dict:
        """
        只在主节点上执行该方法
        """
        if self.should_run():
            self.initial_info_collection_file()
            out = self.cmd('hadoop fsck /')
            parser = TextParser(out.stdout)
            fsck = parser.parse(
                "The filesystem under path '/' is {state}",
                'Total blocks (validated):{:s}{total_blocks:d} {}',
                'Under-replicated blocks:{:s}{urb:d} ({urbp:f} %)',
                'Mis-replicated blocks:{:s}{mrb:d} ({mrbp:f} %)',
                'Corrupt blocks:{:s}{corrupt:d}',
            )
            return fsck

        return {}

    def run(self) -> None:
        fsck = self.fsck()
        if fsck and len(fsck) > 0:
            total_blocks = fsck.get('total_blocks')
            corrupt = fsck.get('corrupt')
            if corrupt > 0:
                self.alarms.add(message.HDFS坏块.format(corrupt, corrupt / total_blocks))
