import message
from core.checker import BaseChecker, share
from core.textparser import TextParser


class HBase(BaseChecker):

    def __init__(self):
        super().__init__('HBase')

    @share
    def hbck(self) -> tuple:
        """
        只在主节点上执行该方法
        """
        if self.should_run():
            self.initial_info_collection_file()
            out = self.cmd('hbase hbck')
            parser = TextParser(out.stdout, strip_header='Summary:')
            cluster_state = parser.parse('{err_count:d} inconsistencies detected.')
            table_state = parser.parse_rows('Table {table} is {status}.')

            return cluster_state, table_state
        return ('', '')

    def run(self) -> None:
        cluster_state, table_state = self.hbck()
        if cluster_state != '':
            err_count = cluster_state.get('err_count')
            if err_count > 0:
                err_tables = [detail['table'] for detail in table_state if detail['status'] == 'inconsistent']
                self.alarms.add(message.HBase不一致.format(err_count, ", ".join(err_tables)))
