import message
from core.checker import BaseChecker, share
from core.textparser import TextParser


class KafkaLog(BaseChecker):

    def __init__(self):
        super().__init__('KafkaLog')

    @share
    def get_topic(self, ret: list) -> list:
        """
        只在主节点上执行该方法
        """
        if self.should_run():
            self.initial_info_collection_file()
            corrupted_topic_list = []
            if ret and len(ret) > 0:
                for item_dict in ret:
                    # 过滤topic
                    topic = item_dict.get('topic').split('/')[5]
                    item_dict['topic'] = topic.split('-')[0]
                    corrupted_topic_list.append(item_dict)

            return corrupted_topic_list

        return []

    def errorlog(self) -> list:
        """
        只在主节点上执行该方法
        """
        if self.should_run():
            self.initial_info_collection_file()
            out = self.cmd(f'cat /var/log/dblog/kafka/server.log | grep -w WARN | grep -w \'Found a corrupted index file\'')

            if out and out.stdout.strip() != '':
                parser = TextParser(out.stdout)
                return parser.parse_rows('[{date}] [WARN] {} index file ({topic}) {}')

        return []

    def run(self) -> None:
        ret = self.errorlog()
        corrupted_topic_list = self.get_topic(ret)
        if corrupted_topic_list and len(corrupted_topic_list) > 0:
            for topic_dict in corrupted_topic_list:
                self.alarms.add(message.Kafka崩溃的Topic.format(topic_dict['date'], topic_dict['topic']))
