import config
import message
from core.checker import BaseChecker, share
from core.textparser import TextParser


class Free(BaseChecker):

    def __init__(self):
        super().__init__('内存')
        self.initial_info_collection_file()

    @share
    def free(self) -> dict:
        out = self.cmd("free -g")
        parser = TextParser(out.stdout)
        free = parser.parse(
            'Mem:{:s}{total:d}{:s}{used:d}{:s}{free:d}{:s}{shared:d}{:s}{cache:d}{:s}{available:d}',
            'Swap:{:s}{swap_total:d}{:s}{swap_used:d}{:s}{swap_free:d}')

        return free

    def run(self) -> None:
        free = self.free()
        swap_total = free['swap_total']
        swap_used = free['swap_used']
        if swap_total > 0:
            self.alarms.add(message.Swap未禁用.format(swap_total, swap_used))
        available = free['available']
        if available < config.memory_min_free_gb:
            self.alarms.add(message.内存剩余不足.format(available, config.memory_min_free_gb))
        total = free['total']
        if total < config.memory_min_total_gb:
            self.alarms.add(message.内存总容量不足.format(total, config.memory_min_total_gb))
