import config
import message
from core.checker import BaseChecker, share
from core.textparser import TextParser
from core.reflection import get_cluster_info


class Ping(BaseChecker):

    def __init__(self):
        super().__init__('网络延迟')
        self.initial_info_collection_file()

    @share
    def pings(self):
        pings = {}
        nodes = list(item['ip'] for item in get_cluster_info().get('nodes'))
        for ip in nodes:
            out = self.cmd(f'ping {ip} -c 100 -i 0 -s 10000 -w 10')
            parser = TextParser(out.stdout)
            ping = parser.parse(
                '{total:d} packets transmitted, {received:d} received, {loss_percent:d}% packet loss, time {time}ms',
                'rtt min/avg/max/mdev = {min:f}/{avg:f}/{max:f}/{mdev:f} ms, ipg/ewma {ipg:f}/{ewma} ms')
            pings[ip] = ping

        return pings

    def run(self) -> None:
        pings = self.pings()
        for ip, detail in pings.items():
            loss_percent = detail['loss_percent']
            avg = detail['avg']
            if loss_percent > 0:
                self.alarms.add(message.网络丢包率高.format(ip, loss_percent))
            if avg > config.network_max_ping_ms:
                self.alarms.add(message.网络延迟高.format(ip, avg, config.network_max_ping_ms))


class SSH(BaseChecker):

    def __init__(self):
        super().__init__('SSH无秘钥')
        self.initial_info_collection_file()

    def run(self) -> None:
        nodes = list(item['ip'] for item in get_cluster_info().get('nodes'))
        ips = self.unavailable_ips(nodes)
        if ips:
            self.alarms.add(message.SSH无秘钥.format(",".join(ips)))

    @share
    def unavailable_ips(self, hosts):
        ips = []
        for ip in hosts:
            out = self.ssh(ip, 'ls')
            if not out.success:
                ips.append(ip)
        return ips
