import datetime
import config
import message
from core.checker import BaseChecker, share
from core.textparser import TextParser


class OS(BaseChecker):

    def __init__(self):
        super().__init__('操作系统')
        self.initial_info_collection_file()

    @share
    def uptime(self) -> datetime:
        """
        获取系统上一次的启动时间
        :param
        :return: 获取系统上一次的启动时间
        """
        out = self.cmd("who -b")
        parser = TextParser(out.stdout)
        rs = parser.parse(
            'system{:s}boot{:s}{date}')
        up_time = datetime.datetime.strptime(f"{rs['date']}", '%Y-%m-%d %H:%M')

        return up_time

    @share
    def arch(self) -> str:
        """
        获取系统架构
        :param
        :return: 返回系统架构
        """
        out = self.cmd('arch')

        return out.stdout.strip()

    @share
    def os(self) -> str:
        """
        获取操作类型
        :param
        :retrun: 返回操作系统类型
        """
        out = self.cmd('cat /etc/redhat-release')
        if not out.stdout.strip():
            out = self.cmd('cat /etc/os-release | grep -w PRETTY_NAME= | cut -d = -f 2 | sed s/\"//g')

        return out.stdout.strip()

    def run(self) -> None:
        # os和arch只需要获取信息
        os = self.os(),
        arch = self.arch()
        up_time = self.uptime()
        up_days = (datetime.datetime.now() - up_time).days
        if up_days < config.uptime_min_days:
            self.alarms.add(message.服务运行时间短.format(up_days))


class Env(BaseChecker):

    def __init__(self):
        super().__init__('环境')
        self.initial_info_collection_file()

    def run(self) -> None:
        cmds = self.missing_cmds(config.cmd_missing)
        if cmds:
            self.alarms.add(message.命令缺失.format(",".join(cmds)))

    @share
    def missing_cmds(self, cmds) -> list:
        """
        通过重点关注的易缺失的Linux命令，查找缺失的系统命令
        :params cmds: 命令列表
        :return: 返回缺失的命令列表
        """
        rs = []
        for cmd in cmds:
            out = self.cmd(f'which {cmd}')
            if not out.success:
                rs.append(cmd)
        return rs
