import config
import message
from checkers.dbserver import DBServer
from core.checker import BaseChecker, CheckerManager, share
from core.textparser import TextParser


class PortCheck(BaseChecker):

    def __init__(self):
        super().__init__('port')
        self.version = None
        self.initial_info_collection_file()

    @share
    def port_check(self) -> dict:
        self.version = CheckerManager.share(DBServer, 'version')
        ret = {}
        parser = TextParser(self.version['exter_version'])
        version = parser.parse_rows('DBE5.0-B33{version:d}.{}', full_match=False)
        ret['port_common'] = self.listen(config.port_common)
        if version[0]['version'] >= int(config.jetty_51):
            ret['jetty_after_51'] = self.listen(config.port_jetty_after_51)
        else:
            ret['jetty_before_51'] = self.listen(config.port_jetty_before_51)

        if version[0]['version'] >= config.yarn_resource_manager_55:
            ret['yarn_resource_manager_after_55'] = self.listen(config.port_yarn_resource_manager_after_55)
        else:
            ret['yarn_resource_manager_before_55'] = self.listen(config.port_yarn_resource_manager_before_55)

        return ret

    def listen(self, port_dict) -> dict:
        rows = {}
        for server, port in port_dict.items():
            out = self.cmd(r"netstat -tunlp | grep " + str(port))
            parser = TextParser(out.stdout)
            row = parser.parse(
                    "{proto}{:s}{recv_q}{:s}{send_q}{:s}{local_address}{:s}{foreign_address}{:s}{state}{:s}{pid}/{program_name}", full_match=False)
            if not row:
                self.alarms.add(message.端口未监听.format(server, port))
            rows.setdefault(server, []).append(row)
        return rows

    def run(self) -> None:
        self.port_check()




