import config
import message
from core.checker import BaseChecker, share
from core.textparser import TextParser


class Process(BaseChecker):

    def __init__(self):
        super().__init__('进程和线程')
        self.initial_info_collection_file()

    @share
    def ps(self) -> list[dict]:
        out = self.cmd('ps -eLf')
        parser = TextParser(
            out.stdout,
            strip_header='UID{:s}PID{:s}PPID{:s}LWP{:s}C{:s}NLWP{:s}STIME{:s}TTY{:s}TIME{:s}CMD')
        ps = parser.parse_rows(
            '{user:S}{:s}{pid}{:s}{ppid}{:s}{LWP:S}{:s}{C}{:s}{NLWP}{:s}{STIME}{:s}{TTY}{:s}{start_time:S}{:s}{cmd}')
        return ps

    def run(self) -> None:
        ps = self.ps()
        thread_num = len(ps)
        if thread_num > config.thread_max_num:
            from collections import Counter
            cmd_count_map = Counter([p['cmd'] for p in ps])
            cmd_count_pairs = sorted(cmd_count_map.items(), key=lambda kv: kv[1], reverse=True)
            top_cmd = '\n'.join([f'{count}个线程： {cmd}' for cmd, count in cmd_count_pairs[:10]])
            self.alarms.add(message.线程数过大.format(thread_num, config.thread_max_num, top_cmd))
