import inspect
import os
from importlib import import_module
from core.cluster import ClusterType
import config


def get_class_by_full_name(full_class_name: str):
    seg = full_class_name.split('.')
    class_name = seg[-1]
    module_name = '.'.join(seg[:-1])
    return get_class(module_name, class_name)


def get_class(module_name, class_name):
    module = import_module(module_name)
    return getattr(module, class_name)


def scan_modules(package):
    for file_name in os.listdir(package):
        path = package + '/' + file_name
        if os.path.isfile(path) and file_name.endswith('.py'):
            module_name = package + '.' + file_name[:-3]
            yield module_name


def scan_class_from_module(module_name, base_class, require_functions=None):
    if require_functions is None:
        require_functions = []
    module = import_module(module_name)
    class_list = inspect.getmembers(module, inspect.isclass)
    for class_name, clazz in class_list:
        if issubclass(clazz, base_class) and clazz != base_class:
            present_func_names = list([name for name, _ in inspect.getmembers(clazz, inspect.isfunction)])
            for require_func in require_functions:
                if require_func not in present_func_names:
                    raise BaseException(f'{class_name}类中必须定义{require_func}方法')
            yield clazz


def scan_class(package, base_class, require_functions=None) -> list:
    for cur_module in scan_modules(package):
        for clazz in scan_class_from_module(cur_module, base_class, require_functions):
            yield clazz


def get_caller_base_on(base_class):
    """
    调用栈中寻找以base_class为基类的对象，仅获取第一个

    :param base_class: 基类
    :return: 符合条件的调用方对象
    """
    for frame in inspect.stack():
        obj = frame[0].f_locals['self']
        if issubclass(obj.__class__, base_class) and obj.__class__ != base_class:
            return obj


def get_cluster_info() -> list[dict]:
    """
    通过反射获取集群节点信息
    """
    cluster_class = get_class_by_full_name(config.cluster_type)
    cluster: ClusterType = cluster_class()
    return cluster.get_cluster_ips()
